using System;
using Zcu.Mve.Core;
using System.IO;
using Zcu.Mve.Strings;
using System.Globalization;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;

namespace Zcu.Mve.Visualization
{
    /// <summary>
    /// Prevede DEM soubor na barevnou vkovou mapu ve formtu JPG.
    /// </summary>
    class DEM2JPG : Module
    {
        private char[] buffer;
        /// <summary>
        /// Jmno zdrojovho souboru.
        /// </summary>
        private string DEMfilename = null;
        /// <summary>
        /// Jmno clovho souboru.
        /// </summary>
        private string JPGfilename = null;
        /// <summary>
        /// Uruje, zda je na vstupu novy soubor. 
        /// </summary>
        private bool newFile = false;
        /// <summary>
        /// Uruje, zda je poteba vytvet nov JPG soubor.
        /// </summary>
        private bool newJPGFile = false;
        /// <summary>
        /// Data naten ze vstupnho souboru.
        /// </summary>
        private UniformDataArray output = null;
        /// <summary>
        /// Poet sloupc v DEM souboru.
        /// </summary>
        private int cols;
        /// <summary>
        /// Ppet dk v DEM souboru.
        /// </summary>
        private int rows;

        public string DEMFileName
        {
            get { return DEMfilename; }
            set
            {
                if (!File.Exists(value))
                {
                    MessageBox.Show("Invalid file name!\n\nClick Cancel or choose existing file.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                this.DEMfilename = value;
                this.newFile = true;
            }
        }

        public string JPGFileName
        {
            get { return JPGfilename; }
            set
            {
                this.JPGfilename = value;
                this.newJPGFile = true;
            }
        }

        /// <summary>
        /// Konstruktor vytvejc vstupn porty.
        /// </summary>
        public DEM2JPG()
        {
            this.AddInPort("DEM filename", typeof(MveString), false);
            this.AddInPort("JPG filename", typeof(MveString), false);
        }

        /// <summary>
        /// Vstupn bod modulu.
        /// </summary>
        public override void Execute(){
            bool newSource = false;
            if (this.GetInPort("DEM filename").Link != null)
            {
                if (!IsDataSame("DEM filename"))
                {
                    this.DEMFileName = ((MveString)GetInput("DEM filename")).Value;
                }
            }
            if (this.GetInPort("JPG filename").Link != null)
            {
                if (!IsDataSame("JPG filename"))
                {
                    this.JPGFileName = ((MveString)GetInput("JPG filename")).Value;
                }
            }
            if (newFile)
            {
                newSource = true;
                newFile = false;
                readPoints();
            }
            if (newSource || newJPGFile)
            {
                newJPGFile = false;
                makeJPG();
            }
        }

        /// <summary>
        /// Nate body z DEM souboru.
        /// </summary>
        private void readPoints() {
            StreamReader reader = new StreamReader(DEMFileName);
            buffer = new char[1024];
            reader.ReadBlock(buffer, 0, 1024);
            int groundUnit = readGroundUnit();
            int elevationUnit = readElevationUnit();
            double[] resolution = readResolution();
            cols = readCols();
            reader.ReadBlock(buffer, 0, 1024);
            rows = readRows();
            output = new UniformDataArray(typeof(Zcu.Mve.Visualization.Point3D), rows * cols);
            int j;
            int pos, x, y, z;
            double[] firstPoint;
            for (int i = 0; i < cols; i++ )
            {
                j = 0;
                pos = 144;
                firstPoint = readFirstPoint();
                while(j != rows ){
                    z =(int) resolution[2] * readNextElevation(pos);
                    x =(int) firstPoint[0];
                    y =(int) (firstPoint[1] + j * resolution[1]); 
                    output[i * rows + j] = new Point3D(x, y, z);
                    pos += 6;
                    j++;
                    if(pos >= 1024 - 5){
                        reader.ReadBlock(buffer, 0, 1024);
                        pos = 0;
                    }
                }
                reader.ReadBlock(buffer, 0, 1024);
            }
            reader.Close();
        }

        /// <summary>
        /// Pete jednotky pouit pro souadnice X a Y.
        /// </summary>
        /// <returns>Kd danch jednotek.</returns>
        private int readGroundUnit() {
            return Int32.Parse(new String(buffer, 529, 6));
        }

        /// <summary>
        /// Pete jednotky pouit pro souadnici Z.
        /// </summary>
        /// <returns>Kd dan jednotky.</returns>
        private int readElevationUnit() {
            return Int32.Parse(new String(buffer, 535, 6));
        }

        /// <summary>
        /// Pete rozlien ve vech smrech.
        /// </summary>
        /// <returns>Pole odpovdajc danmu rozlien.</returns>
        private double[] readResolution() {
            double[] ret = new double[3];
            NumberFormatInfo nfi = new CultureInfo("en-US", false).NumberFormat;
            
            ret[0] = Double.Parse(new String(buffer, 816, 12), NumberStyles.Float, nfi);
            ret[1] = Double.Parse(new String(buffer, 828, 12), NumberStyles.Float, nfi);
            ret[2] = Double.Parse(new String(buffer, 840, 12), NumberStyles.Float, nfi);

            return ret;
        }

        /// <summary>
        /// Pete poet sloupc..
        /// </summary>
        /// <returns>Poet sloupc.</returns>
        private int readCols() {
            return Int32.Parse(new String(buffer, 858, 6));
        }

        /// <summary>
        /// Pete poet dk.
        /// </summary>
        /// <returns>Poet dk.</returns>
        private int readRows() {
            return Int32.Parse(new String(buffer, 12, 6));
        }

        /// <summary>
        /// Nate souadnici Z.
        /// </summary>
        /// <param name="pos">Pozice v souboru, kde dan souadnice zan.</param>
        /// <returns>Souadnice Z.</returns>
        private int readNextElevation(int pos) {
            return Int32.Parse(new String(buffer, pos, 6));
        }

        /// <summary>
        /// Pete souadnice X a Y prvnho bodu v danm sloupci.
        /// </summary>
        /// <returns>Souadnice X a Y.</returns>
        private double[] readFirstPoint() {
            double[] ret = new double[2];
            NumberFormatInfo nfi = new CultureInfo("en-US", false).NumberFormat;

            ret[0] = Double.Parse(new String(buffer, 24, 20), NumberStyles.Float, nfi) * Math.Pow(10, Int32.Parse(new String(buffer, 45, 3)));
            ret[1] = Double.Parse(new String(buffer, 24, 20), NumberStyles.Float, nfi) * Math.Pow(10, Int32.Parse(new String(buffer, 69, 3)));

            return ret;
        }

        /// <summary>
        /// Vytvo vkovou mapu z natench dat.
        /// </summary>
        private void makeJPG(){
            Bitmap bitmap = new Bitmap(cols, rows);
            
            for (int x = 0; x < cols; x++ )
            {
                for (int y = 0; y < rows; y++ )
                {
                    byte hue = (byte) ((-2.0 / 100) * (( Point3D )output[x * rows + y]).Z + 150);
                    HSBColor c = new HSBColor(255, hue, 255, 255);
                    bitmap.SetPixel(x, y , c);
                }
            }
            bitmap.Save(JPGFileName, ImageFormat.Jpeg);
        }
    }
}
