using System;
using System.IO;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using Zcu.Mve.Strings;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization
{
    /// <summary>
    /// Tda pro naten bod z PLY souboru.
    /// </summary>
    class PLYLoader : Module
    {
        /// <summary>
        /// Jmno vstupnho souboru.
        /// </summary>
        private string filename = null;
        /// <summary>
        /// Uruje zmny vstupnho souboru.
        /// </summary>
        private bool newFile = false;
        /// <summary>
        /// Vstupn data.
        /// </summary>
        private UniformDataArray output = null;

        public string FileName {
            get { return filename; }
            set {
                if (!File.Exists(value))         
                {
                    MessageBox.Show("Invalid file name!\n\nClick Cancel or choose existing file.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                this.filename = value;
                this.newFile = true;
            }
        }

        /// <summary>
        /// Konstruktor vytvejc porty.
        /// </summary>
        public PLYLoader(){
            this.AddInPort("filename", typeof(MveString), false);
            this.AddOutPort("Point3D array", typeof(UniformDataArray));
        }

        /// <summary>
        /// Vstupn bod modulu.
        /// </summary>
        public override void Execute()
        {
            if (this.GetInPort("filename").Link != null)
            {
                if (!IsAllDataSame())
                {
                    this.FileName = ((MveString)GetInput("filename")).Value;
                }
            }
            if (newFile)
            {
                newFile = false;
                readPoints();
                SetOutput("Point3D array", output);
            }
        }

        /// <summary>
        /// Nate body ze vstupnho souboru.
        /// </summary>
        private void readPoints()
        {
            StreamReader reader = new StreamReader(filename);
            string line;
            while (!(line = reader.ReadLine()).Contains("element vertex"));
            line = line.Substring(14);
            int rows = Int32.Parse(line);
            while (!(line = reader.ReadLine()).Contains("end_header")) ;
            string[] s;
            output = new UniformDataArray(typeof(Zcu.Mve.Visualization.Point3D), rows);
            Regex regex = new Regex(@"\s+");
            for (int i = 0; i < rows; i++)
            {
                line = reader.ReadLine().Trim();
                s = regex.Split(line);
                output[i] = new Zcu.Mve.Visualization.Point3D(Int32.Parse(s[0]), Int32.Parse(s[1]), (s.Length < 3)? 0 : Int32.Parse(s[2]));
            }
            reader.Close();
        }
    }
}
