using System;
using System.IO;
using Zcu.Mve.Core;
using Zcu.Mve.Strings;

namespace Zcu.Mve.Visualization
{
    /// <summary>
    /// Tda pro uloen bod do PLY souboru.
    /// </summary>
    class PLYSaver : Module
    {
        /// <summary>
        /// Jmno vstupnho souboru.
        /// </summary>
        private string filename = null;
        /// <summary>
        /// Data uren k zapsn.
        /// </summary>
        private UniformDataArray data = null;
        /// <summary>
        /// Uruje zmnu nzvu vtupnho souboru.
        /// </summary>
        private bool newFile = false;

        public string FileName
        {
            get { return filename; }
            set
            {
                this.filename = value;
                this.newFile = true;
            }
        }

        /// <summary>
        /// Konstruktor vytvejc porty.
        /// </summary>
        public PLYSaver()
        {
            this.AddInPort("Point3D array", typeof(UniformDataArray));
            this.AddInPort("filename", typeof(MveString), false);        
        }

        /// <summary>
        /// Vstupn bod modulu. Zape data do souboru.
        /// </summary>
        public override void Execute()
        {
            if (!IsAllDataSame())
            {
                if (this.GetInPort("filename").Link != null)
                {
                    this.FileName = ((MveString)GetInput("filename")).Value;
                }
                newFile = true; 
            }
            if (newFile)
            {
                newFile = false;
                data = (UniformDataArray)GetInput("Point3D array");
                StreamWriter writer = new StreamWriter(FileName);
                writer.WriteLine("ply");
                writer.WriteLine("format ascii 1.0");
                writer.WriteLine("element vertex " + data.Count);
                writer.WriteLine("property float x");
                writer.WriteLine("property float y");
                writer.WriteLine("property float z");
                writer.WriteLine("end_header");
                for (int i = 0; i < data.Count; i++)
                {
                    writer.Write(((Point3D)data[i]).X + " ");
                    writer.Write(((Point3D)data[i]).Y + " ");
                    writer.WriteLine(((Point3D)data[i]).Z);
                }
                writer.Close();
            }
        }
    }
}
