using System;
using System.IO;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using Zcu.Mve.Strings;
using Zcu.Mve.Core;


namespace Zcu.Mve.Visualization
{
    /// <summary>
    /// Tda pro naten bod z PTS souboru.
    /// </summary>
    class PTSLoader : Module
    {
        /// <summary>
        /// Jmno souboru.
        /// </summary>
        private string filename = null;
        /// <summary>
        /// Uruje zmnu nzvu vstupnho souboru.
        /// </summary>
        private bool newFile = false;
        /// <summary>
        /// Vstupn data.
        /// </summary>
        private UniformDataArray output = null;

        public string FileName {
            get { return filename; }
            set {
                if (!File.Exists(value))         
                {
                    MessageBox.Show("Invalid file name!\n\nClick Cancel or choose existing file.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                this.filename = value;
                this.newFile = true;
            }
        }

        /// <summary>
        /// Konstuktor vytvejc porty.
        /// </summary>
        public PTSLoader(){
            this.AddInPort("filename", typeof(MveString), false);
            this.AddOutPort("Point3D array", typeof(UniformDataArray));
        }

        /// <summary>
        /// Vstupn bod modulu.
        /// </summary>
        public override void Execute()
        {
            if (this.GetInPort("filename").Link != null)
            {
                if (!IsAllDataSame())
                {
                    this.FileName = ((MveString)GetInput("filename")).Value;
                }
            }
            if (newFile)
            {
                newFile = false;
                readPoints();
                SetOutput("Point3D array", output);
            }
        }

        /// <summary>
        /// Nate body ze souboru.
        /// </summary>
        private void readPoints() {
            StreamReader reader = new StreamReader(FileName);
            string line = reader.ReadLine();
            string[] s;
            int rows = Int32.Parse(line);
            output = new UniformDataArray(typeof(Point3D),rows);
            Regex regex = new Regex(@"\s+");
            for (int i = 0; i < rows; i++ ) {
                line = reader.ReadLine().Trim();
                s = regex.Split(line);
                output[i] = new Point3D(Int32.Parse(s[0]), Int32.Parse(s[1]), (s.Length < 3)? 0 : Int32.Parse(s[2]));
            }
            reader.Close();
        }
    }
}
